/*
  crashRect
      by John (Crashed) Souto

  Using crashRect is very simple. It was mainly designed for a pixel movement library that I'm
  going to write (or perhaps all ready have, depends if I have updated this since then :P).

  procedures:
    rect(x,y,width,height):
      Creates a new rect object and returns the reference.

  datums:
    rect:
      variables:
        x:
          The left-most position of the rectangle.
        y:
          The bottom-most position of the rectangle.
        width:
          How wide the rectangle is.
        height:
          How tall the rectangle is.
      functions:
        New(x,y,width,height,add):
          Initializes all the variables. If add is true, it will add the rectangle to the
          RECT_MASTER_LIST.
        collide(rect):
          See if two rectangles collide. rect may be a list of arguments (for example,
          list(64,65,12,54) would be a rectangle with an x value of 64, y of 65, width of 12,
          and height of 54) or another rect object. Returns true if collision is detected, false
          if not.
        move(x,y,check):
          Move relatively to a given coordinate. x and y are how many pixels to shift over to.
          If check is set to true, the rectangle will not move if another rectangle is occupying
          that space. Returns true if movement was successful, false if not.
        move_to(x,y,check):
          Move to a given coordinate. x and y are how many pixels to move to. If check is set
          to true, the rectangle will not move if another rectangle is occupying that space.
          Returns true if movement was successful, false if not.
        change(width,height,check):
          Change the relative size of the rectangle. width and height are the amount of pixels to
          add or subtract. If check is set to true, the rectangle will not change if another
          rectangle is occupying that space. Returns true if change was successful, false if not.
        change_to(width,height,check):
          Change the size of the rectangle. width and height are the amount of pixels to add or
          subtract. If check is set to true, the rectangle will not change if another rectangle
          if occupying that space. Returns true if change was successful, false if not.
*/

proc
	rect(x,y,width,height)
		return new/rect(x,y,width,height)

var
	list
		RECT_MASTER_LIST=list()

rect
	var
		x=0
		y=0
		width=0
		height=0

	New(x,y,width,height,add=TRUE)
		src.x=x
		src.y=y
		src.width=width
		src.height=height
		if(add)
			RECT_MASTER_LIST+=src

	Del()
		if(src in RECT_MASTER_LIST)
			RECT_MASTER_LIST-=src
		..()

	proc
		collide(rect/rect2)
			if(istype(rect2,/list))
				rect2=rect(arglist(rect2))
			if((x<=(rect2.x+rect2.width)&&x>=rect2.x)||((x+width)<=(rect2.x+rect2.width)&&(x+width)>=rect2.x))
				if((y<=(rect2.y+rect2.height)&&y>=rect2.y)||((y+height)<=(rect2.y+rect2.height)&&(y+height)>=rect2.y))
					return TRUE
			return FALSE
		move(x=0,y=0,check=TRUE)
			x+=src.x
			y+=src.y
			if(check)
				for(var/rect/rect in RECT_MASTER_LIST)
					if(rect!=src&&rect.collide(rect(x,y,width,height)))
						return FALSE
			src.x=x
			src.y=y
			return TRUE
		move_to(x=0,y=0,check=TRUE)
			if(check)
				for(var/rect/rect in RECT_MASTER_LIST)
					if(rect!=src&&rect.collide(rect(x,y,width,height)))
						return FALSE
			src.x=x
			src.y=y
			return TRUE
		change(width=0,height=0,check=TRUE)
			width+=src.width
			height+=src.height
			if(check)
				for(var/rect/rect in RECT_MASTER_LIST)
					if(rect!=src&&rect.collide(rect(x,y,width,height)))
						return FALSE
			src.width=width
			src.height=height
			return TRUE
		change_to(width=0,height=0,check=TRUE)
			if(check)
				for(var/rect/rect in RECT_MASTER_LIST)
					if(rect!=src&&rect.collide(rect(x,y,width,height)))
						return FALSE
			src.width=width
			src.height=height
			return TRUE